--[[
BaseSingletonClass = {_singleton = nil}
BaseSingletonClass.new = function(self)
	local class = {}
	setmetatable(class, self)
	self.__index = self
	return class
end
BaseSingletonClass.getSingleton = function(self)
	if self._singleton == nil then
		self._singleton = self:new()
	end
	return self._singleton
end
BaseSingletonClass.purgeSingleton = function(self)
	if self._singleton ~= nil then
		self._singleton = nil
		self:delete()
	end
end
]]

stMontorName = {
	initWithStream = function(dt)
		if dt == nil then
			return {}
		end
		local tmp = {}
		tmp.ucType = dt:readByte()
		tmp.nId = dt:readInt()
		tmp.ucAmount = dt:readByte()
		tmp.strName = {}
		for i=1, tmp.ucAmount do
			local str = dt:readString()
			if str then
				table.insert(tmp.strName, str:getData())
			else
				table.insert(tmp.strName, " ")
			end
		end
		
		return tmp
	end
}
stRaidsAward = {
	initWithStream = function(dt)
		if dt == nil then
			return {}
		end
		local tmp = {}
		tmp.ucType = dt:readByte()
		tmp.ucTurns = dt:readByte()
		tmp.ucStrap = dt:readByte()
		tmp.dwEXP = dt:readInt()
		tmp.dwGodEXP = dt:readInt()
		tmp.dwMoney = dt:readInt()
		tmp.szItem = " "
		local str = dt:readString()
		if str then
			tmp.szItem = str:getData()
		end
		tmp.szNames = " "
		local str = dt:readString()
		if str then
			tmp.szNames = str:getData()
		end
		tmp.dwEMoney = dt:readInt()
		
		return tmp
	end
}

RaidsCopyProcessor = {_singleton}
RaidsCopyController = {_singleton}

--RaidsCopyProcessor类
RaidsCopyProcessor.getSingleton = function(self)
	if self._singleton == nil then
		self._singleton = self:new()
		self._singleton:init()
	end
	return self._singleton
end
RaidsCopyProcessor.purgeSingleton = function(self)
	if self._singleton ~= nil then
		self:delete()
	end
end
RaidsCopyProcessor.new = function(self)
	local class = {m_copyType=0, m_id=0}
	setmetatable(class, self)
	self.__index = self
	return class
end
RaidsCopyProcessor.delete = function(self)
	self._singleton = nil
	self.m_copyType = 0
	self.m_id = 0
end
RaidsCopyProcessor.init = function(self)
	LuaData:getSingleton():luaSetSelectorOnRecvData(RaidsCopyProcessor.onMsgMonstorName, LuaData_ACTION_COPY_MONSTOR_NAME)
	LuaData:getSingleton():luaSetSelectorOnRecvData(RaidsCopyProcessor.onMsgStopRaids, LuaData_ACTION_COPY_WIPE_END)
end
RaidsCopyProcessor.onMsgMonstorName = function()
	local dt = LuaData:getSingleton():getDataTransStream()
    if dt == nil then
        return
    end
	RaidsCopyProcessor:getSingleton():showRaidsCopyWindow(dt);
end
RaidsCopyProcessor.onMsgStopRaids = function()
	local dt = LuaData:getSingleton():getDataTransStream()
    if dt == nil then
        return
    end
	local ctrl = getCurentSceneWindowController(5801)
	if not ctrl then
		ctrl = getCurentSceneWindowController(5802)
	end
	if not ctrl then
		return
	end
	RaidsCopyController:getSingleton():stopRaids()
end
RaidsCopyProcessor.showRaidsCopyWindow = function(self, dataStream)
	local ctrl = RaidsCopyController:getSingleton():newRaidsCopyCtrl(dataStream)
	if not isWindControllerExistence(5801) and not isWindControllerExistence(5802) and ctrl then
		getCurentSceneNavigationController():pushWindowController(ctrl)
	end
end
RaidsCopyProcessor.setCopyType = function(self, copyType)
	self.m_copyType = copyType
end
RaidsCopyProcessor.getCopyType = function(self)
	return self.m_copyType
end
RaidsCopyProcessor.setIdTemplate = function(self, id)
	self.m_id = id
end
RaidsCopyProcessor.getIdTemplate = function(self)
	return self.m_id
end
RaidsCopyProcessor.sendStartRaid = function(self, turns, isAutoSell)
	local ACTION_COPYWIPE_MONSTOR_AWARD = 18
	local dataStream = SCDataTransStream:streamWithCommand(_MSG_INSTANCE)
	dataStream:writeShort(ACTION_COPYWIPE_MONSTOR_AWARD)
	dataStream:writeByte(self.m_copyType)
	dataStream:writeInt(self.m_id)
	dataStream:writeByte(turns)
	if isAutoSell then
		dataStream:writeByte(1)
	else
		dataStream:writeByte(0)
	end
	SCDataTransThread:getSingleton():transData(dataStream)
	SCActivityIndicator:getSingleton():show()
end
RaidsCopyProcessor.sendOpenRaid = function(self, copyType, idTemplate)
	local ACTION_COPY_MONSTOR_NAME = 17
	local dataStream = SCDataTransStream:streamWithCommand(_MSG_INSTANCE)
	dataStream:writeShort(ACTION_COPY_MONSTOR_NAME)
	dataStream:writeByte(copyType)
	dataStream:writeInt(idTemplate)
	SCDataTransThread:getSingleton():transData(dataStream)
	SCActivityIndicator:getSingleton():show()
end
RaidsCopyProcessor.sendStopRaid = function(self)
	local action = 23
	if self.m_copyType == 2 then
		action = 22
	end
	local dataStream = SCDataTransStream:streamWithCommand(_MSG_INSTANCE)
	dataStream:writeShort(action)
	SCDataTransThread:getSingleton():transData(dataStream)
	SCActivityIndicator:getSingleton():show()
end

--RaidsCopyController类
RaidsCopyController.getSingleton = function(self)
	if self._singleton == nil then
		self._singleton = self:new()
		self._singleton:init()
	end
	return self._singleton
end
RaidsCopyController.purgeSingleton = function(self)
	if self._singleton ~= nil then
		self:delete()
	end
end
RaidsCopyController.new = function(self)
	local class = {m_turns=0, m_id=0, m_isAutoSell=false, m_seleteIndex=0, m_wnd=nil}
	setmetatable(class, self)
	self.__index = self
	return class
end
RaidsCopyController.delete = function(self)
	self._singleton = nil
	self.m_turns = 0
	self.m_id = 0
	self.m_isAutoSell = false
	self.m_seleteIndex = 0
	self.m_wnd = nil
end
RaidsCopyController.init = function(self)

end
RaidsCopyController.newRaidsCopyCtrl = function(self, dataStream)
	local ctrl = self:initWithCopyType(dataStream)
	if ctrl then
		return ctrl
	end
	return nil
end
RaidsCopyController.initWithCopyType = function(self, dataStream)
	local montorName = stMontorName.initWithStream(dataStream)
	RaidsCopyProcessor:getSingleton():setCopyType(montorName.ucType)
	RaidsCopyProcessor:getSingleton():setIdTemplate(montorName.nId)
	local name = 5801
	local copyType = RaidsCopyProcessor:getSingleton():getCopyType()
	if copyType == 2 then
		name = 5802
	end
	local pWnd = SCTypeCast:castNodeToWindow(SCFunction:getUiById(name))
	self.m_wnd = pWnd
	local wndCtrl = SCWindowController:controllerWithWindow(pWnd)
	if not wndCtrl then
		return nil
	end
	local imgPath = getRealUIResourcePath("img/titile50.png")
	if imgPath and pWnd then
		pWnd:setTitleWithImage(imgPath:getData())
	end
	pWnd:luaSetSelectorOnClose(self.onWndClose)

	local pDragNode = SCTypeCast:castNodeToDragNode(SCFunction:getUiNodeByIdlistAndParent("1", pWnd))
	if pDragNode then
		pDragNode:setFrameType(EnumFrameTypeRound)
		pDragNode:setFrameVisibled(true)
	end
	pDragNode = SCTypeCast:castNodeToDragNode(SCFunction:getUiNodeByIdlistAndParent("2", pWnd))
	if pDragNode then
		pDragNode:setFrameType(EnumFrameTypeRound)
		pDragNode:setFrameVisibled(true)
	end

	local pLabel = SCTypeCast:castNodeToLabel(SCFunction:getUiNodeByIdlistAndParent("3", pWnd))
	if pLabel then
		pLabel:setColor(color_lightYellow)
	end
	pLabel = SCTypeCast:castNodeToLabel(SCFunction:getUiNodeByIdlistAndParent("4", pWnd))
	if pLabel then
		pLabel:setColor(color_gray)
	end

	local pListBox = SCTypeCast:castNodeToListBox(SCFunction:getUiNodeByIdlistAndParent("7", pWnd))
	if pListBox then
		for i=1, montorName.ucAmount do
			local pParserNode = getItemTextNode(montorName.strName[i])
			pListBox:addItem(pParserNode)
		end
	end


	local pCheckBox = SCTypeCast:castNodeToCheckBox(SCFunction:getUiNodeByIdlistAndParent("8", pWnd))
	if pCheckBox then
		pCheckBox:setDescribe(_LTParser("auto sell"), getDefaultFontName(), getFontSize(12), color_lightYellow)
	end

	if copyType == 2 then
		local tmpTurn = 5
		local tiredness = Hero:getSingleton():getTiredness() + Hero:getSingleton():getTirednessAdd()
		if tiredness < 25 then
			tmpTurn = tiredness/5
		end
		self:setTurn(tmpTurn)
		local pRadioGroup = SCTypeCast:castNodeToRadioGroup(SCFunction:getUiNodeByIdlistAndParent("10", pWnd))
		if pRadioGroup then
			pRadioGroup:setDescribe(0, _LTParser("need how many tiredness")..(tmpTurn*5).._LTParser("str0108"), getDefaultFontName(), getFontSize(12), color_lightYellow)
			pRadioGroup:setDescribe(1, _LTParser("use all tiredness"), getDefaultFontName(), getFontSize(12), color_lightYellow)
			pRadioGroup:luaSetSelectorOnItemSelected(RaidsCopyController.onRadioGroupItemSelected)
			pRadioGroup:luaSetSelectorOnItemUnSelected(RaidsCopyController.onRadioGroupItemUnSelected)
		end

		local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlistAndParent("11", pWnd))
		if pEdit then
			pEdit:setIsNumberText(true)
			pEdit:setContentText(tmpTurn.."")
			pEdit:setBackgroundColor(color_transparent)
			--pEdit:setContentSize(CCSizeMake(SX(32), SY(16)))
			pEdit:luaSetSelectorOnBeginInput(RaidsCopyController.onEditBeginInput)
			pEdit:luaSetSelectorOnEndInput(RaidsCopyController.onEditEndInput)
			pEdit:luaSetSelectorOnInsertText(RaidsCopyController.onEditInsertText)
			pEdit:setBottomColor(color_transparent)
			pEdit:setFontColor(color_white)
			pWnd:luaSetSelectorOnTouched(RaidsCopyController.onWindowTouched)
		end
	end

	local pBtn = SCTypeCast:castNodeToButton(SCFunction:getUiNodeByIdlistAndParent("9", pWnd))
	if pBtn then
		pBtn:luaSetSelectorOnClicked(RaidsCopyController.onClickStartRaids)
	end
	
	local pTextBox = SCTypeCast:castNodeToTextBox(SCFunction:getUiNodeByIdlistAndParent("5", pWnd))
	if pTextBox then
		pTextBox:setText("")
	end

	return wndCtrl
end
RaidsCopyController.addAwardInfo = function(self, dataStream)
	local pBtn = SCTypeCast:castNodeToButton(SCFunction:getUiNodeByIdlistAndParent("9", self.m_wnd))
	local pTextBox = SCTypeCast:castNodeToTextBox(SCFunction:getUiNodeByIdlistAndParent("5", self.m_wnd))
	if not pBtn or not pTextBox then
		return
	end
	pBtn:setTitle(_LTParser("stop raids"), getDefaultFontName(), getFontSize(12))
	pBtn:luaSetSelectorOnClicked(RaidsCopyController.onClickStopRaids)

	local text = self:makeText(dataStream)


	local count = pTextBox:getLineCount()


	for i=1, count-30 do
		pTextBox:removeTopLine()
	end

	pTextBox:addText(text)
	pTextBox:setOffset(CCPointMake(0, pTextBox:getContentSize().height))
end
RaidsCopyController.stopRaids = function(self)
	local pBtn = SCTypeCast:castNodeToButton(SCFunction:getUiNodeByIdlistAndParent("9", self.m_wnd))
	if pBtn then
		pBtn:setTitle(_LTParser("start raids"), getDefaultFontName(), getFontSize(12))
		pBtn:luaSetSelectorOnClicked(RaidsCopyController.onClickStartRaids)
	end
end
RaidsCopyController.onClickStartRaids = function(tag)
	local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
	if pEdit then
		RaidsCopyController:getSingleton().m_turns = pEdit:getContentText()+0
	end
	local pCheckBox = SCTypeCast:castNodeToCheckBox(SCFunction:getUiNodeByIdlistAndParent("8", RaidsCopyController:getSingleton().m_wnd))
	if pCheckBox then
		RaidsCopyController:getSingleton().m_isAutoSell = pCheckBox:isSelect();
		RaidsCopyProcessor:getSingleton():sendStartRaid(RaidsCopyController:getSingleton().m_turns, RaidsCopyController:getSingleton().m_isAutoSell)
	end
end
RaidsCopyController.onClickStopRaids = function(tag)
	RaidsCopyProcessor:getSingleton():sendStopRaid()
	local pBtn = SCTypeCast:castNodeToButton(SCFunction:getUiNodeByIdlistAndParent("9", RaidsCopyController:getSingleton().m_wnd))
	if pBtn then
		pBtn:setTitle(_LTParser("start raids"), getDefaultFontName(), getFontSize(12))
		pBtn:luaSetSelectorOnClicked(RaidsCopyController.onClickStartRaids)
	end
end
RaidsCopyController.onWndClose = function(tag)
	RaidsCopyProcessor:getSingleton():sendStopRaid()
	if RaidsCopyProcessor:getSingleton():getCopyType()==3 then
		local dataStream = SCDataTransStream:streamWithCommand(_MSG_INSTANCE)
		dataStream:writeShort(10)
		dataStream:writeByte(3)
		SCDataTransThread:getSingleton():transData(dataStream)
		SCActivityIndicator:getSingleton():show()
	end
	refresh_EquiptMakeScreen()
	RaidsCopyController:getSingleton():delete()
end
RaidsCopyController.onEditBeginInput = function(tag)
	local pRadioGroup = SCTypeCast:castNodeToRadioGroup(SCFunction:getUiNodeByIdlist("5802-10"))
	if pRadioGroup then
		pRadioGroup:setSelectIndex(0)
	end
end
RaidsCopyController.onEditEndInput = function(tag)
	local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
	if not pEdit then
		return
	end
	local text = pEdit:getContentText()
	if not tonumber(text) then
		pEdit:setContentText("")
		return
	end
	local turn = tonumber(text)
	local tiredness = Hero:getSingleton():getTiredness() + Hero:getSingleton():getTirednessAdd()
	if turn > tiredness/5 then
		pEdit:setContentText((tiredness/5).."")
		RaidsCopyController:getSingleton():setTurn(tiredness/5)
	else
		RaidsCopyController:getSingleton():setTurn(turn)
	end

	if pEdit:getIsInputing() then
		pEdit:detachWithIME()
	end
end
RaidsCopyController.onEditInsertText = function(tag, tlen, text)
	local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
	if not pEdit then
		return true
	end
	local strOldText = pEdit:getContentText()
	local strInsertText = text
	if strInsertText == "\n" then
		return false
	end
	if strOldText=="0" then
		pEdit:setContentText("")
	end
	strOldText = strOldText..strInsertText
	local turn = tonumber(strOldText)
	local tiredness = Hero:getSingleton():getTiredness() + Hero:getSingleton():getTirednessAdd()
	if turn > tiredness/5 then
		pEdit:setContentText(tiredness/5)
		RaidsCopyController:getSingleton():setTurn(tiredness/5)
		return true
	else
		RaidsCopyController:getSingleton():setTurn(turn)
		return false
	end
end
RaidsCopyController.onWindowTouched = function(tag)
	local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
	if pEdit and pEdit:getIsInputing() then
		pEdit:detachWithIME()
	end
end
RaidsCopyController.onRadioGroupItemSelected = function(tag, itemIndex)
	RaidsCopyController:getSingleton().m_seleteIndex = itemIndex;
	if itemIndex == 1 then
		local tiredness = Hero:getSingleton():getTiredness() + Hero:getSingleton():getTirednessAdd();
		RaidsCopyController:getSingleton():setTurn(tiredness/5);
		local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
		if pEdit then
			pEdit:setContentText(RaidsCopyController:getSingleton().m_turns.."")
			if pEdit:getIsInputing() then
				pEdit:detachWithIME()
			end
		end
	else
		local tmpTurn = 5;
		local tiredness = Hero:getSingleton():getTiredness() + Hero:getSingleton():getTirednessAdd();
		if tiredness < 25 then
			tmpTurn = tiredness/5;
		end
		RaidsCopyController:getSingleton():setTurn(tmpTurn);
		local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlist("5802-11"))
		if pEdit then
			pEdit:setContentText(RaidsCopyController:getSingleton().m_turns.."")
			if pEdit:getIsInputing() then
				pEdit:detachWithIME()
			end
		end
	end
end
RaidsCopyController.onRadioGroupItemUnSelected = function(tag, index)
	
end
RaidsCopyController.setTurn = function(self, turn)
	self.m_turns = turn;
	local pRadioGroup = SCTypeCast:castNodeToRadioGroup(SCFunction:getUiNodeByIdlistAndParent("10", self.m_wnd))
	if pRadioGroup then
		pRadioGroup:setDescribe(0, _LTParser("need how many tiredness")..(self.m_turns*5).._LTParser("point"), getDefaultFontName(), getFontSize(12), color_lightYellow);
	end
end
RaidsCopyController.makeText = function(self, dataStream)
	local str = "";
	local raidsAward = stRaidsAward.initWithStream(dataStream);
	if raidsAward.ucStrap == 1 then
		str = str..string.format(_LTParser("str5095"), raidsAward.ucTurns)
	end
	if raidsAward.ucStrap == 0 then
		if RaidsCopyProcessor:getSingleton():getCopyType()==2 then
			if self.m_turns>0 then
				self.m_turns =  self.m_turns - 1;
				local pEdit = SCTypeCast:castNodeToEdit(SCFunction:getUiNodeByIdlistAndParent("11", self.m_wnd))
				if pEdit then
					pEdit:setContentText(self.m_turns);
				end
				self:setTurn(self.m_turns);
			end
		end
		
		str = str.._LTParser("str0109");
	else
		str = str.._LTParser("str0110");
		str = str..raidsAward.ucStrap;
		str = str.."_*0";
	end
	str = str..raidsAward.szNames;
	str = str.._LTParser("str0111");
	str = str..raidsAward.dwEXP;

	if raidsAward.ucStrap == 0 then
		str = str.._LTParser("str0112");
	end
	if raidsAward.dwGodEXP ~= 0 then
		str = str.._LTParser("str0113");
		str = str..raidsAward.dwGodEXP;
	end
	if raidsAward.dwMoney ~= 0 then
		str = str.._LTParser("str0114");
		str = str..raidsAward.dwMoney;
	end
	if raidsAward.dwEMoney ~= 0 then
		str = str.._LTParser("str0303");
		str = str..raidsAward.dwEMoney;
	end
	
	str = str.._LTParser("str0115");
	str = str..raidsAward.szItem;
	str = str.."_ _";
	if raidsAward.ucStrap == 0 then
		str = str..string.format(_LTParser("str5125"), raidsAward.ucTurns)
		str = str.."_ _ "
	end
	return str;
end

RaidsCopyProcessor:getSingleton()